// ==============================================================
// ORBITER vessel module: Biconic Crew Exploration Vehicle (CEV2)
// by francisdrake, 2005
// ==============================================================

#include "orbitersdk.h"
#include <stdio.h>

#define R_SIZE	2.5, 0.25	// RCS flame size
#define FR_SIZE	1.2, 0.12	// front RCS flame size

#define MAIN_ENG_1  _V( 1.78, 0,-3.4)	// main engine location
#define MAIN_ENG_2  _V(-1.78, 0,-3.4)
#define MAIN_ENGSIZE 7, 1.2			// main engine flame size

#define RCS_THRUST		  500		// RM RCS thrust
#define FRCS_THRUST		  186		// front RCS thrust

#define RCS_ISP			 3840		// main engine and RCS have same ISP	
#define MAIN_THRUST	 	 9000		// N
		


// PARTICLESTREAMSPEC typedef struct {
// flags, srcsize, srcrate, v0, srcspread, lifetime, growthrate, atmslowdown, LTYPE, 
// LEVELMAP, lmin, lmax,
// ATMSMAP, amin, amax,
// tex };

PARTICLESTREAMSPEC main_stream = {
	0, 0.6, 60, 60.0, 0.1, 0.04, 6, 0, PARTICLESTREAMSPEC::EMISSIVE,
	PARTICLESTREAMSPEC::LVL_PSQRT, 0.0, 0.05,
	PARTICLESTREAMSPEC::ATM_FLAT, 0.1, 0.1
	};

PARTICLESTREAMSPEC land_stream = {
	0, 3, 60, 2.0, 0.1, 0.7, 6, 10, PARTICLESTREAMSPEC::DIFFUSE,
	PARTICLESTREAMSPEC::LVL_PSQRT, 0.0, 0.05,
	PARTICLESTREAMSPEC::ATM_FLAT, 0.1, 0.1
	};


// ==========================================================
// Interface for derived vessel class: CEV
// ==========================================================

class CEV: public VESSEL2 {
	friend BOOL CALLBACK RMS_DlgProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
public:
	CEV (OBJHANDLE hObj, int fmodel);
	void SetConfig1_Orbit (void);
	void SetConfig2_Reentry (void);
	void SetConfig3_Drogue (void);
	void SetConfig4_Chute (void);
	void SetConfig5_Chute_deployed(void);
	void SetConfig6_Land(void);

	void SetThrusters1_Orbit(void);
	void SetThrusters2_Reentry(void);
	void SetThrusters3_Land(void);
	void ReloadMeshes(void);

	void do_eva();
	
	// CEV specific varaiables
	PROPELLANT_HANDLE ph_RM, ph_CM;
	int mode;		// 0 = launch config, (CM) + ressource module (RM)
					// 1 = orbit config, CM + RM
					// 2 = reentry config, CM only
					// 3 = drogue config, CM + drogue chute
					// 4 = chute config 1, reefed
					// 5 = chute config 2, fully deployed
					// 6 = landed
	
	int  solar;		// 0 = solar panel closed
					// 1 = solar panel deployed

	int cl_mult;	// multiply the lift of the chute
	char string [256];

	VECTOR3 ofs;
	MESHHANDLE hAstroMesh1, hAstroMesh2;

	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);

private:
	SURFHANDLE tex_rcs, tex_main;
};

// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
//	static const double CL[nabsc] = {0.1,  0.17, 0.2,  0.2,  0.17, 0.1,  0,    -0.11, -0.24, -0.3,  -0.3,  -0.3, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CL[nabsc] = {0.05, 0.08, 0.09, 0.09, 0.08, 0.05, 0.00, -0.05, -0.11, -0.18, -0.24, -0.24, -0.01, 0.30, 0.30, 0.22, 0.13, 0.06, 0.00, -0.08, -0.12, -0.14, -0.11, -0.05, 0.05};
		
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.002,0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 0.2 + oapiGetInducedDrag (*cl, 1.5, 0.3);
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0, 0.2, 0.3, 0.2, 0, -0.2, -0.3, -0.2, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.06 + oapiGetInducedDrag (*cl, 1.5, 0.3);
}


void VLiftCoeff_drogue (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
//	static const double CL[nabsc] = {0.1,  0.17, 0.2,  0.2,  0.17, 0.1,  0,    -0.11, -0.24, -0.3,  -0.3,  -0.3, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CL[nabsc] = {0.05, 0.08, 0.09, 0.09, 0.08, 0.05, 0.00, -0.05, -0.11, -0.18, -0.24, -0.24, -0.01, 0.30, 0.30, 0.22, 0.13, 0.06, 0.00, -0.08, -0.12, -0.14, -0.11, -0.05, 0.05};
		
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.004,0.008, 0.005,0.0024,      0,-0.0024,-0.0014,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = CM[idx] + (CM[idx+1]-CM[idx])*d;
	*cd = 1 + oapiGetInducedDrag (*cl, 1.5, 0.3);
}

void VLiftCoeff_chute1 (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
//	static const double CL[nabsc] = {0.1,  0.17, 0.2,  0.2,  0.17, 0.1,  0,    -0.11, -0.24, -0.3,  -0.3,  -0.3, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CL[nabsc] = {0.0,  0.00, 0.00, 0.00, 0.00, 0.00, 0.00, -0.00, -0.00, -0.00, -0.00, -0.00,  0.5, 2.00, 2.50, 0.22, 0.00, 0.00, 0.00, -0.00, -0.00, -0.00, -0.00, -0.00, 0.00};
		
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.004,0.008, 0.005,0.0024,      0,-0.0024,-0.0014,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = 1 * (CL[idx] + (CL[idx+1]-CL[idx])*d);	// Multiplied to make up for reduced area
	*cm = 0.5*(CM[idx] + (CM[idx+1]-CM[idx])*d);
	*cd = 0.7 + oapiGetInducedDrag (*cl, 15, 0.5);
}

void VLiftCoeff_chute2 (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
//	static const double CL[nabsc] = {0.1,  0.17, 0.2,  0.2,  0.17, 0.1,  0,    -0.11, -0.24, -0.3,  -0.3,  -0.3, -0.02, 0.6355,    0.63,   0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CL[nabsc] = {0.0,  0.00, 0.00, 0.00, 0.00, 0.00, 0.00, -0.00, -0.00, -0.00, -0.00, -0.00,  0.5, 2.00, 2.50, 0.22, 0.00, 0.00, 0.00, -0.00, -0.00, -0.00, -0.00, -0.00, 0.00};
		
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,    0,0.004,0.008, 0.005,0.0024,      0,-0.0024,-0.0014,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = 4 * (CL[idx] + (CL[idx+1]-CL[idx])*d);	// Multiplied to make up for reduced area
	*cm = 0.5*(CM[idx] + (CM[idx+1]-CM[idx])*d);
	*cd = 0.7 + oapiGetInducedDrag (*cl, 15, 0.5);
}

// ==============================================================
// Specialised vessel class
// ==============================================================

// Constructor
CEV::CEV (OBJHANDLE hObj, int fmodel)
: VESSEL2(hObj, fmodel)
{
	mode  = 0;
	solar = 0;
	cl_mult = 1;
	ofs   = _V(0,0,0);

	SetConfig1_Orbit();

	tex_rcs  = oapiRegisterExhaustTexture ("exhaust_crcs");
	tex_main = oapiRegisterExhaustTexture ("exhaust_crcs"); 

	hAstroMesh1 = oapiLoadMeshGlobal ("CEV2\\CEV2-astro");
}

// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new CEV (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (CEV*) vessel;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	((CEV *) vessel)-> SetConfig1_Orbit();
	((CEV *) vessel)-> SetCameraRotationRange(PI, PI, 0.4*PI, 0.4*PI);
}

//===============================
// Read status from scenario file
//===============================
void CEV::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	char *line;
	int value;

	while (oapiReadScenario_nextline (scn, line)) {
        if (!strnicmp (line, "MODE", 4)) {
            sscanf (line+4, "%d", &value);
			 mode = (int) value;
		} else if (!strnicmp (line, "SOLAR", 5)) {
            sscanf (line+5, "%d", &value);
			 solar = (int) value;
		} else { 
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
	}

	switch (mode) {
	case 0:
	case 1:
		SetConfig1_Orbit ();
		break;
	case 2:
		SetConfig2_Reentry();
		break;
	case 3:
		SetConfig3_Drogue();
		break;
	case 4:
	case 5:
		SetConfig4_Chute();
		break;
	case 6:
		SetConfig6_Land();
		break;
	}
}

void CEV::clbkSaveState (FILEHANDLE scn)
{	// default vessel parameters
	SaveDefaultState (scn);	
	// custom parameters
	oapiWriteScenario_int (scn, "MODE", mode);	
	oapiWriteScenario_int (scn, "SOLAR", solar);		
}


void CEV::clbkPostStep (double simt, double simdt, double mjd)
{
//	sprintf (oapiDebugString(), "Mode= %d", mode);
	
	if (mode > 2)  {	
		if (oapiGetTimeAcceleration() > 10)		// Slow down
			oapiSetTimeAcceleration(10);

//			AddForce(_V(0,0,-1000), _V(0,3,0));

			if ((mode == 4 && GetAirspeed() < 50)) {	// fully deploy chute
				CreateAirfoil (LIFT_VERTICAL,   _V(0,5,-1), VLiftCoeff_chute2, 16, 0, 0.6);
				mode = 5;
			}	

		// Check if we shall cut the chute
		if (GetAltitude() < 3.5 && mode <6 ) {
		// Create the chute as an individual object
		VESSELSTATUS vs;
		GetStatus (vs);
		vs.flag[0] = 3;
		VECTOR3 ofs = {0,0,2};
		Local2Rel (ofs, vs.rpos);

		VECTOR3 vel = _V(0,30,-30);
		VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
		Local2Rel (ofs, vs.rpos);
		GlobalRot (vel, rofs);
		vs.rvel.x = rvel.x+rofs.x;
		vs.rvel.y = rvel.y+rofs.y;
		vs.rvel.z = rvel.z+rofs.z;

		char name[256];
		strcpy (name, GetName()); strcat (name, "-Chute");
		oapiCreateVessel (name, "CEV2\\CEV2-Chute", vs);

		SetConfig6_Land();
		}
	}
}

int CEV::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events

//	VESSEL *sh = (VESSEL*)vessel;
	
	if (KEYMOD_SHIFT (kstate)) {
	} else {						// unmodified keys
		switch (key) {
		case OAPI_KEY_J: // Jettison service module
			if ( mode < 2) {

			if (oapiGetTimeAcceleration() > 10)		// Slow down
				oapiSetTimeAcceleration(10);

			// Create Ressource Module as an individual object
			VESSELSTATUS vs;
			GetStatus (vs);
			vs.flag[0] = 3;
			VECTOR3 ofs = {0,0,0};
			Local2Rel (ofs, vs.rpos);

			VECTOR3 vel = _V(0,0,-0.2);
			VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
			Local2Rel (ofs, vs.rpos);
			GlobalRot (vel, rofs);
			vs.rvel.x = rvel.x+rofs.x;
			vs.rvel.y = rvel.y+rofs.y;
			vs.rvel.z = rvel.z+rofs.z;

			char name[256]= {"aaa"};
			strcpy (name, GetName()); strcat (name, "-RM");
			oapiCreateVessel (name, "CEV2\\CEV2-RM", vs);

			// Create belly as an individual object
			GetStatus (vs);
			vs.flag[0] = 3;
			Local2Rel (ofs, vs.rpos);

			vel = _V(0,-0.2,-0.05);
			rofs = _V(vs.rvel.x, vs.rvel.y, vs.rvel.z); 
			rvel = _V(vs.rvel.x, vs.rvel.y, vs.rvel.z);
			Local2Rel (ofs, vs.rpos);
			GlobalRot (vel, rofs);
			vs.rvel.x = rvel.x+rofs.x;
			vs.rvel.y = rvel.y+rofs.y;
			vs.rvel.z = rvel.z+rofs.z;

			strcpy (name, GetName()); strcat (name, "-belly");
			oapiCreateVessel (name, "CEV2\\CEV2-belly", vs);

// Hack to display the solar panel around the separated RM
			solar = 1;	// deploy solar panels in case not yet open
			GetStatus (vs);
			vs.flag[0] = 3;			
			ofs = _V(0,0,0);
	
			Local2Rel (ofs, vs.rpos);

			vel = _V(0,0,-0.2);
			rofs, rvel = _V(vs.rvel.x, vs.rvel.y, vs.rvel.z);
			Local2Rel (ofs, vs.rpos);
			GlobalRot (vel, rofs);
			vs.rvel.x = rvel.x+rofs.x;
			vs.rvel.y = rvel.y+rofs.y;
			vs.rvel.z = rvel.z+rofs.z;
			strcat (name, "S");							 
		 	oapiCreateVessel (name, "CEV2\\CEV2-RMS", vs); // config file of created solar panel

			SetConfig2_Reentry();
			}
			return 1;	
			
			
// Deploy drogue and chute
			case OAPI_KEY_C: 

			double speed;
			char name[256];
			OBJHANDLE hvessel;			

			if (GetAltitude() > 40000)
				return 1;		// dont open the chute above 40 km

			strcpy (name, GetName()); 
			hvessel=oapiGetVesselByName(name);
			oapiGetAirspeed(hvessel, &speed);

			if (( mode  == 2) && (speed < 1000)) {
				SetConfig3_Drogue();

			// Create back as an individual object
			VESSELSTATUS vs;
			GetStatus (vs);
			vs.flag[0] = 3;
			VECTOR3 ofs = {0,0,0};
			Local2Rel (ofs, vs.rpos);

			VECTOR3 vel = _V(0,0.5,-1);
			VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
			Local2Rel (ofs, vs.rpos);
			GlobalRot (vel, rofs);
			vs.rvel.x = rvel.x+rofs.x;
			vs.rvel.y = rvel.y+rofs.y;
			vs.rvel.z = rvel.z+rofs.z;

			char name[256]= {"aaa"};
			strcpy (name, GetName()); strcat (name, "-back");
			oapiCreateVessel (name, "CEV2\\CEV2-back", vs);

			return 1;
			} 
			
			if ((mode == 3) && (speed < 120)) {
			SetConfig4_Chute();
			
			// Create the drogue as an individual object
			VESSELSTATUS vs;
			GetStatus (vs);
			vs.flag[0] = 3;
			VECTOR3 ofs = {0,0,0};
			Local2Rel (ofs, vs.rpos);

			VECTOR3 vel = _V(0,0,-2);
			VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
			Local2Rel (ofs, vs.rpos);
			GlobalRot (vel, rofs);
			vs.rvel.x = rvel.x+rofs.x;
			vs.rvel.y = rvel.y+rofs.y;
			vs.rvel.z = rvel.z+rofs.z;

			char name[256];
			strcpy (name, GetName()); strcat (name, "-Drogue");
			oapiCreateVessel (name, "CEV2\\CEV2-Drogue", vs);
			}
			return 1;

		case OAPI_KEY_E: 
			do_eva();
			return 1;

		case OAPI_KEY_S: // Deploy solar panels
			if (mode < 2)	{	// use while in orbit mode only
				solar = 1;
				ReloadMeshes();
			}
			return 1;
		} // end switch
	} // end else
	return 0;
}


//===========================
void CEV::SetConfig1_Orbit(void)
{
	SetSize (7);
	SetEmptyMass (12000);
	SetPMI (_V(9,9,4));

	SetCW (0.2, 0.5, 1, 1);
	SetCrossSections (_V(40,40,24));
	SetRotDrag (_V(0.3,0.3,0.2));
	
	SetCOG_elev (2.5);

	SetPitchMomentScale (1e-4);
	SetBankMomentScale  (1e-4);

	SetDockParams (_V(0,-0.15,-4), _V(0,0,-1), _V(0,1,0));
	SetCameraOffset (_V(-0.85,1,-1.8));

	SetThrusters1_Orbit();
	mode = 1;
	ReloadMeshes();
}

void CEV::SetConfig2_Reentry(void)
{
	SetSize (7);
	SetEmptyMass (8000);
	SetPMI (_V(9,9,4));
	SetTrimScale (0.04);
	SetCW (0.2, 0.5, 1, 1);
	SetCrossSections (_V(26,28,23));
	SetRotDrag (_V(0.3,0.3,0.2));
	
	SetTouchdownPoints(_V(0,-1.9,5),_V(-3,-3,0),_V(3,-3,0));
	SetSurfaceFrictionCoeff (0.2, 0.2);

	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 7, 0, 1.4);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-0.5), HLiftCoeff, 7, 0, 1.4);

	CreateControlSurface (AIRCTRL_ELEVATOR,     3.0, 0.75, _V( 0, -1.8,  -7), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 3.0, 0.75, _V( 0, -1.8,  -7), AIRCTRL_AXIS_XPOS);
//	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  0.5, 0.75, _V( 1.5,-1.8,-7), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  0.5, 0.75, _V(-1.5,-1.8,-7), AIRCTRL_AXIS_XNEG);

	SetADCtrlMode (7);

	SetThrusters2_Reentry();
	mode = 2;
	ReloadMeshes();
}

void CEV::SetConfig3_Drogue(void)
{
	SetSize (7);
	SetEmptyMass (8000.0);
	SetPMI (_V(9,9,4));
	SetTrimScale (0.04);
	SetCW (0.5, 0.5, 1, 1);
	SetCrossSections (_V(26,28,23));
	SetRotDrag (_V(0.3,0.3,0.2));	

	SetTouchdownPoints(_V(0,-1.9,5),_V(-3,-3,0),_V(3,-3,0));
	SetSurfaceFrictionCoeff (0.2, 0.2);
	
	CreateAirfoil (LIFT_VERTICAL,   _V(0,1,-0.5), VLiftCoeff_drogue, 7, 0, 1.4);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,1,-0.5), HLiftCoeff, 7, 0, 1.4);

	CreateControlSurface (AIRCTRL_ELEVATOR,     3.0, 0.75, _V( 0, -1.8,  -7), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 3.0, 0.75, _V( 0, -1.8,  -7), AIRCTRL_AXIS_XPOS);
//	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  0.5, 0.75, _V( 1.5,-1.8,-7), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  0.5, 0.75, _V(-1.5,-1.8,-7), AIRCTRL_AXIS_XNEG);
	
//	SetThrusters2_Reentry();
	ClearThrusterDefinitions();
	mode = 3;
	ReloadMeshes();
}

void CEV::SetConfig4_Chute(void)
{
	SetSize (18);
	SetEmptyMass (8000.0);
	SetPMI (_V(9,9,4));
	SetTrimScale (0.04);
	SetCW (0.2, 0.5, 1, 1);
	SetCrossSections (_V(40, 100,40));
	SetRotDrag (_V(0.3,0.3,0.45));

	SetTouchdownPoints(_V(0,-1.9,5),_V(-3,-3,0),_V(3,-3,0));
	SetSurfaceFrictionCoeff (0.3, 0.3);

	CreateAirfoil (LIFT_VERTICAL,   _V(0,5,-1), VLiftCoeff_chute1, 16, 0, 0.6);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,5,-1), HLiftCoeff, 16, 0, 0.6);

	CreateControlSurface (AIRCTRL_ELEVATOR,     40.0, 0.75, _V( 0, 0,  -7), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM,  5.0, 0.75, _V( 0, 0,  -7), AIRCTRL_AXIS_XPOS);
//	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  5, 0.75, _V( 15,-1.8,-7), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  5, 0.75, _V(-15,-1.8,-7), AIRCTRL_AXIS_XNEG);
	
//	SetThrusters2_Reentry();
	ClearThrusterDefinitions();
	mode = 4;
	ReloadMeshes();
}

void CEV::SetConfig5_Chute_deployed(void)
{	
	SetConfig4_Chute();
	CreateAirfoil (LIFT_VERTICAL,   _V(0,5,-1), VLiftCoeff_chute2, 16, 0, 0.6);
	mode = 5;
}

void CEV::SetConfig6_Land(void)
{
	SetSize (7);
	SetEmptyMass (8000.0);
	SetPMI (_V(9,9,4));
	ClearAirfoilDefinitions();

	SetTouchdownPoints(_V(0,-1.9,5),_V(-3,-3,0),_V(3,-3,0));
	SetSurfaceFrictionCoeff (0.3, 0.3);

	SetThrusters3_Land();

	mode = 6;
	ReloadMeshes();
}

void CEV::ReloadMeshes(void)
{
	ClearMeshes();
	SetCameraOffset (_V(-1,0.8, 1));	// left hand seat, top window

	ofs = _V(-0.85, 0.1, 2.25);
	AddMesh (hAstroMesh1, &ofs);

//	ofs = _V( 0.85, 0.1, 2.25);
//	AddMesh (hAstroMesh1, &ofs);

	AddMesh ("CEV2\\CEV2-CM");	// Always reender the CM

	switch (mode) {
	case 0:
	case 1:		// Orbit config
		AddMesh ("CEV2\\CEV2-RM");
		AddMesh ("CEV2\\CEV2-belly");
		AddMesh ("CEV2\\CEV2-back");

		if (solar)	
			AddMesh ("CEV2\\CEV2-solar1");
		else 
			AddMesh ("CEV2\\CEV2-solar0");
		break;

	case 2:		// Reentry config
		AddMesh ("CEV2\\CEV2-back");
		break;	// capsule is already rendered

	case 3:		// Drogue config
			SetMeshVisibilityMode(AddMesh ("CEV2\\CEV2-drogue"), MESHVIS_ALWAYS);
		break;
	case 4:		// not used
	case 5:		// Chute config
		SetMeshVisibilityMode(AddMesh ("CEV2\\CEV2-chute"), MESHVIS_ALWAYS);
		AddMesh ("CEV2\\CEV2-gear");
		break;
	case 6:		// Landed config
		AddMesh ("CEV2\\CEV2-gear");
		break;

	}
}

//===============================
// Create thrusters for the resource module
// ===============================

void CEV::SetThrusters1_Orbit()
{
	ClearThrusterDefinitions();
	ClearPropellantResources();
	ph_RM = CreatePropellantResource (5500);	 // RM propellant
	SetDefaultPropellantResource(ph_RM);

	THRUSTER_HANDLE th_att_rot[16], th_att_lin[16];
// Pitchup front
	th_att_rot[0] = th_att_lin[0] =  CreateThruster (_V(1.01,-0.04,5.09), _V(-0.2, 1, 0), FRCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[1] = th_att_lin[1] =  CreateThruster (_V(0.93,-0.05,5.31), _V(-0.2, 1, 0), FRCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[2] = th_att_lin[2] =  CreateThruster (_V(-1.01,-0.04,5.09), _V(0.2, 1,0), FRCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[3] = th_att_lin[3] =  CreateThruster (_V(-0.93,-0.05,5.31), _V(0.2, 1,0), FRCS_THRUST, ph_RM, RCS_ISP);
	
// Pitchup RM
	th_att_rot[4] = th_att_lin[8] =  CreateThruster (_V(2.85,0.31,-2.07), _V(-0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[5] = th_att_lin[9] =  CreateThruster (_V(2.85,0.31,-1.79), _V(-0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[6] = th_att_lin[10] =  CreateThruster (_V(-2.85,0.31,-2.07), _V(0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[7] = th_att_lin[11] =  CreateThruster (_V(-2.85,0.31,-1.79), _V(0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	
// Pitchdown front
	th_att_rot[8] = th_att_lin[12] =  CreateThruster (_V(0.92,0.38,5.09), _V(-0.2, -1, 0), FRCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[9] = th_att_lin[13] =  CreateThruster (_V(0.84,0.38,5.31), _V(-0.2, -1, 0), FRCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[10] = th_att_lin[14] =  CreateThruster (_V(-0.92,0.38,5.09), _V(0.2, -1, 0), FRCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[11] = th_att_lin[15] =  CreateThruster (_V(-0.84,0.38,5.31), _V(0.2, -1, 0), FRCS_THRUST, ph_RM, RCS_ISP);
	
// Pitchdown RM
	th_att_rot[12] = th_att_lin[4] =  CreateThruster (_V(2.88,-0.31,-1.57), _V(-0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[13] = th_att_lin[5] =  CreateThruster (_V(2.88,-0.31,-2.32), _V(-0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[14] = th_att_lin[6] =  CreateThruster (_V(-2.88,-0.31,-1.57), _V(0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[15] = th_att_lin[7] =  CreateThruster (_V(-2.88,-0.31,-2.32), _V(0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);

	CreateThrusterGroup (th_att_rot,   8, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+8, 8, THGROUP_ATT_PITCHDOWN);

	CreateThrusterGroup (th_att_lin,   8, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+8, 8, THGROUP_ATT_DOWN);

	for (int i=0;i<4;i++)
		AddExhaust (th_att_rot[i], FR_SIZE, tex_rcs);
	for (i=4;i<8;i++)
		AddExhaust (th_att_rot[i], R_SIZE, tex_rcs);
	for (i=8;i<12;i++)
		AddExhaust (th_att_rot[i], FR_SIZE, tex_rcs);
	for (i=12;i<16;i++)
		AddExhaust (th_att_rot[i], R_SIZE, tex_rcs);

// Bankleft RM only
	th_att_rot[0] =  CreateThruster (_V(2.88,-0.31,-1.56), _V(-0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[1] =  CreateThruster (_V(2.88,-0.31,-2.32), _V(-0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[2] =  CreateThruster (_V(-2.85,0.31,-2.07), _V(0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[3] =  CreateThruster (_V(-2.85,0.31,-1.79), _V(0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);

// Bankright RM only
	th_att_rot[4] =  CreateThruster (_V(-2.88,-0.31,-1.56), _V(0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[5] =  CreateThruster (_V(-2.88,-0.31,-2.32), _V(0.2, 1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[6] =  CreateThruster (_V(2.85,0.31,-2.07), _V(-0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[7] =  CreateThruster (_V(2.85,0.31,-1.79), _V(-0.2, -1, 0), RCS_THRUST, ph_RM, RCS_ISP);

	CreateThrusterGroup (th_att_rot,   4, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+4, 4, THGROUP_ATT_BANKRIGHT);

	for (i=0;i<8;i++)
		AddExhaust (th_att_rot[i], R_SIZE, tex_rcs);
	
//Attleft front
	th_att_lin[0] =  CreateThruster (_V(0.99,0.17,5.09), _V(-1, -0.17, 0), RCS_THRUST/10.9, ph_RM, RCS_ISP);
	th_att_lin[1] =  CreateThruster (_V(0.90,0.15,5.32), _V(-1, -0.17, 0), RCS_THRUST/10.9, ph_RM, RCS_ISP);

//Attleft back
	th_att_lin[2] =  CreateThruster (_V(2.55,0.44,-0.36), _V(-1, -0.17, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_lin[3] =  CreateThruster (_V(2.61,0.44,-0.59), _V(-1, -0.17, 0), RCS_THRUST, ph_RM, RCS_ISP);
	
//Attright front
	th_att_lin[4] =  CreateThruster (_V(-0.99,0.17,5.09), _V(1, -0.17, 0), RCS_THRUST/10.9, ph_RM, RCS_ISP);
	th_att_lin[5] =  CreateThruster (_V(-0.90,0.15,5.32), _V(1, -0.17, 0), RCS_THRUST/10.9, ph_RM, RCS_ISP);

//Attright back
	th_att_lin[6] =  CreateThruster (_V(-2.55,0.44,-0.36), _V(1, -0.17, 0), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_lin[7] =  CreateThruster (_V(-2.61,0.44,-0.59), _V(1, -0.17, 0), RCS_THRUST, ph_RM, RCS_ISP);

	CreateThrusterGroup (th_att_lin,   4, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+4, 4, THGROUP_ATT_RIGHT);

	for (i=0;i<2;i++)
		AddExhaust (th_att_lin[i], FR_SIZE, tex_rcs);
	for (i=2;i<4;i++)
		AddExhaust (th_att_lin[i], R_SIZE, tex_rcs);
	for (i=4;i<6;i++)
		AddExhaust (th_att_lin[i], FR_SIZE, tex_rcs);
	for (i=6;i<8;i++)
		AddExhaust (th_att_lin[i], R_SIZE, tex_rcs);

// Yawleft RM
//Forward
	th_att_rot[0] = th_att_lin[0] =  CreateThruster (_V(2.87, 0.12,-2.61), _V(-0.2, 0, 1), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[1] = th_att_lin[1] =  CreateThruster (_V(2.87,-0.12,-2.61), _V(-0.2, 0, 1), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[4] = th_att_lin[2] =  CreateThruster (_V(-2.87, 0.12,-2.61), _V(0.2, 0, 1), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[5] = th_att_lin[3] =  CreateThruster (_V(-2.87,-0.12,-2.61), _V(0.2, 0, 1), RCS_THRUST, ph_RM, RCS_ISP);

//Back
	th_att_rot[6] = th_att_lin[4] =  CreateThruster (_V(2.87, 0.12,-1.30), _V(-0.2, 0, -1), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[7] = th_att_lin[5] =  CreateThruster (_V(2.87,-0.12,-1.30), _V(-0.2, 0, -1), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[2] = th_att_lin[6] =  CreateThruster (_V(-2.87, 0.12,-1.30), _V(0.2, 0, -1), RCS_THRUST, ph_RM, RCS_ISP);
	th_att_rot[3] = th_att_lin[7] =  CreateThruster (_V(-2.87,-0.12,-1.30), _V(0.2, 0, -1), RCS_THRUST, ph_RM, RCS_ISP);

	CreateThrusterGroup (th_att_lin,   4, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+4, 4, THGROUP_ATT_BACK);

	CreateThrusterGroup (th_att_rot, 4, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+4, 4, THGROUP_ATT_YAWRIGHT);

	for (i=0;i<8;i++)
		AddExhaust (th_att_lin[i], R_SIZE, tex_rcs);

// main
	th_att_lin[0] =  CreateThruster (MAIN_ENG_1, _V(0,0, 1), MAIN_THRUST, ph_RM, RCS_ISP);
	th_att_lin[1] =  CreateThruster (MAIN_ENG_2, _V(0,0, 1), MAIN_THRUST, ph_RM, RCS_ISP);
// main dummy for exhaust stream
	th_att_lin[2] =  CreateThruster (_V( 1.8, 0,-4), _V(0,0, 1), 0, ph_RM, RCS_ISP);
	th_att_lin[3] =  CreateThruster (_V(-1.8, 0,-4), _V(0,0, 1), 0, ph_RM, RCS_ISP);

	CreateThrusterGroup (th_att_lin, 4, THGROUP_MAIN);

	AddExhaust (th_att_lin[0], MAIN_ENGSIZE, MAIN_ENG_1, _V( 0, 0, -1), tex_main);
	AddExhaust (th_att_lin[1], MAIN_ENGSIZE, MAIN_ENG_2, _V( 0, 0, -1), tex_main);

	AddExhaustStream(th_att_lin[2], &main_stream);
	AddExhaustStream(th_att_lin[3], &main_stream);
}

//=======================================
// Create thrusters for the capsule alone
// ======================================
void CEV::SetThrusters2_Reentry()
{
	ClearThrusterDefinitions();
	ClearPropellantResources();

	ph_CM = CreatePropellantResource (320);	 // CM propellant
	SetDefaultPropellantResource(ph_CM);
	
	THRUSTER_HANDLE th_att_rot[16], th_att_lin[16];
// Pitchup front
	th_att_rot[0] = th_att_lin[0] =  CreateThruster (_V(1.01,-0.04,5.09), _V(-0.2, 1, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[1] = th_att_lin[1] =  CreateThruster (_V(0.93,-0.05,5.31), _V(-0.2, 1, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[2] = th_att_lin[2] =  CreateThruster (_V(-1.01,-0.04,5.09), _V(0.2, 1,0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[3] = th_att_lin[3] =  CreateThruster (_V(-0.93,-0.05,5.31), _V(0.2, 1,0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	
// Pitchup back
	th_att_rot[4] = th_att_lin[8] =  CreateThruster (_V(2.54,0.66,-0.36), _V(-0.2, -1, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[5] = th_att_lin[9] =  CreateThruster (_V(2.60,0.66,-0.58), _V(-0.2, -1, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[6] = th_att_lin[10] =  CreateThruster (_V(-2.54,0.66,-0.36), _V(0.2, -1, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[7] = th_att_lin[11] =  CreateThruster (_V(-2.60,0.66,-0.58), _V(0.2, -1, 0), RCS_THRUST, ph_CM, RCS_ISP);
	
// Pitchdown front
	th_att_rot[8] = th_att_lin[12] =  CreateThruster (_V(0.90,0.41,5.09), _V(-0.2, -1, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[9] = th_att_lin[13] =  CreateThruster (_V(0.82,0.41,5.32), _V(-0.2, -1, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[10] = th_att_lin[14] =  CreateThruster (_V(-0.90,0.41,5.09), _V(0.2, -1, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[11] = th_att_lin[15] =  CreateThruster (_V(-0.82,0.41,5.32), _V(0.2, -1, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	
// Pitchdown back
	th_att_rot[12] = th_att_lin[4] =  CreateThruster (_V(2.68,0.21,-0.59), _V(-0.4, 1, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[13] = th_att_lin[5] =  CreateThruster (_V(2.63,0.19,-0.35), _V(-0.4, 1, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[14] = th_att_lin[6] =  CreateThruster (_V(-2.68,0.21,-0.59), _V(0.4, 1, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[15] = th_att_lin[7] =  CreateThruster (_V(-2.63,0.19,-0.35), _V(0.4, 1, 0), RCS_THRUST, ph_CM, RCS_ISP);

	CreateThrusterGroup (th_att_rot,   8, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+8, 8, THGROUP_ATT_PITCHDOWN);

	CreateThrusterGroup (th_att_lin,   8, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+8, 8, THGROUP_ATT_DOWN);

	for (int i=0;i<16;i++)
		AddExhaust (th_att_rot[i], FR_SIZE, tex_rcs);

// Bankleft back
	th_att_rot[0] =  CreateThruster (_V(2.68,0.21,-0.59), _V(-0.4, 1, 0), FRCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[1] =  CreateThruster (_V(2.63,0.19,-0.35), _V(-0.4, 1, 0), FRCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[2] =  CreateThruster (_V(-2.54,0.66,-0.36), _V(0.2, -1, 0), FRCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[3] =  CreateThruster (_V(-2.60,0.66,-0.58), _V(0.2, -1, 0), FRCS_THRUST, ph_CM, RCS_ISP);

// Bankright back
	th_att_rot[4] =  CreateThruster (_V(-2.68,0.21,-0.59), _V(0.4, 1, 0), FRCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[5] =  CreateThruster (_V(-2.63,0.19,-0.35), _V(0.4, 1, 0), FRCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[6] =  CreateThruster (_V(2.54,0.66,-0.36), _V(-0.2, -1, 0), FRCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[7] =  CreateThruster (_V(2.60,0.66,-0.58), _V(-0.2, -1, 0), FRCS_THRUST, ph_CM, RCS_ISP);

	CreateThrusterGroup (th_att_rot,   4, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+4, 4, THGROUP_ATT_BANKRIGHT);

	for (i=0;i<8;i++)
		AddExhaust (th_att_rot[i], FR_SIZE, tex_rcs);

//Yawleft front
	th_att_rot[0] = th_att_lin[0] =  CreateThruster (_V(0.99,0.17,5.09), _V(-1, -0.17, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[1] = th_att_lin[1] =  CreateThruster (_V(0.90,0.15,5.32), _V(-1, -0.17, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);

//Yawleft back
	th_att_rot[2] = th_att_lin[4] =  CreateThruster (_V(-2.55,0.44,-0.36), _V(1, -0.17, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[3] = th_att_lin[5] =  CreateThruster (_V(-2.61,0.44,-0.59), _V(1, -0.17, 0), RCS_THRUST, ph_CM, RCS_ISP);
	
//Yawright front
	th_att_rot[4] = th_att_lin[6] =  CreateThruster (_V(-0.99,0.17,5.09), _V(1, -0.17, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);
	th_att_rot[5] = th_att_lin[7] =  CreateThruster (_V(-0.90,0.15,5.32), _V(1, -0.17, 0), RCS_THRUST/10.9, ph_CM, RCS_ISP);

//Yawright back
	th_att_rot[6] = th_att_lin[2] =  CreateThruster (_V(2.55,0.44,-0.36), _V(-1, -0.17, 0), RCS_THRUST, ph_CM, RCS_ISP);
	th_att_rot[7] = th_att_lin[3] =  CreateThruster (_V(2.61,0.44,-0.59), _V(-1, -0.17, 0), RCS_THRUST, ph_CM, RCS_ISP);
	
	CreateThrusterGroup (th_att_rot, 4, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+4, 4, THGROUP_ATT_YAWRIGHT);

	CreateThrusterGroup (th_att_lin,   4, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+4, 4, THGROUP_ATT_RIGHT);

	for (i=0;i<8;i++)
		AddExhaust (th_att_rot[i], FR_SIZE, tex_rcs);
}

//=======================================
// Create thrusters for the landing
// ======================================
void CEV::SetThrusters3_Land()
{
//	ClearThrusterDefinitions();
//	ClearPropellantResources();

	ph_CM = CreatePropellantResource (20);	 // ground braking propellant
	
	THRUSTER_HANDLE th_att_lin[16];

	th_att_lin[0] =  CreateThruster (_V(2,-2,-1), _V(-0.2, 1, 0), 10000, ph_CM, RCS_ISP);
	th_att_lin[1] =  CreateThruster (_V(2,-2,2), _V(-0.2, 1, 0), 5000, ph_CM, RCS_ISP);
	th_att_lin[2] =  CreateThruster (_V(-2,-2,-1), _V(0.2, 1, 0), 10000, ph_CM, RCS_ISP);
	th_att_lin[3] =  CreateThruster (_V(-2,-2,2), _V(0.2, 1, 0), 5000, ph_CM, RCS_ISP);

	CreateThrusterGroup (th_att_lin,   4, THGROUP_ATT_UP);
	SetThrusterGroupLevel(THGROUP_ATT_UP, 1);

	for (int i=0;i<4;i++)
		AddExhaustStream(th_att_lin[i], &land_stream);
}

void CEV::do_eva()		{
			static char name[256];
			static OBJHANDLE hvessel, hEVA;

			strcpy (name, GetName()); strcat (name, "-EVA");
			hvessel=oapiGetVesselByName(name);
			if (hvessel == 0)
			{
				// Separate EVA astronaut
				VESSELSTATUS vs;
				GetStatus (vs);
				char name[256];

				vs.flag[0] = 3;
				VECTOR3 ofs = {0,2.7,1.5};
				VECTOR3 rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				
				VECTOR3 vel = {0,0.1,0};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, ofs);
				vs.rvel.x = rvel.x+ofs.x;
				vs.rvel.y = rvel.y+ofs.y;
				vs.rvel.z = rvel.z+ofs.z;

				vs.fuel = 1;

				strcpy (name, GetName()); strcat (name, "-EVA");
				hEVA = VESSEL::Create (name, "CEV2\\CEV2-Astro", vs);
				hvessel=oapiGetVesselByName(name);
				if (hvessel != 0) {
					oapiSetFocusObject(hvessel);
				}
					
			} else {
//				hvessel=oapiGetVesselByName(name);
//				if (hvessel != 0)
					oapiSetFocusObject(hvessel);
			}
}
